package org.jworkplace.login;

import java.util.*;
import java.io.IOException;
import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;


public class WorkPlaceLoginModule implements LoginModule {
    //initial state
    private Subject subject;
    private CallbackHandler callbackHandler;
    private Map sharedState;
    private Map options;

    //configurable option
    private boolean debug = false;

    //the authentication status
    private boolean succeeded = false;
    private boolean commitSucceeded = false;

    //username and password
    private String username;
    private char[] password;
    ;

    private WorkPlacePrincipal userPrincipal;

    //the initialize method is called by the LoginContext
    public void initialize(Subject subject, CallbackHandler callbackHandler, 
                           Map sharedState, Map options) {
        this.subject = subject;
        this.callbackHandler = callbackHandler;
        this.sharedState = sharedState;
        this.options = options;
    }

    //this is the login method that is called by the LoginContext
    public boolean login() throws LoginException {
        //prompt for a username and password
        if (callbackHandler == null) {
            throw new LoginException("Error:no CallbackHandler available");
        }

        //Two Callback are created,one for the name and one for the password
        Callback[] callbacks = new Callback[2];
        callbacks[0] = new NameCallback("JWorkPlace username:");
        callbacks[1] = new PasswordCallback("JWorkPlace password:", false);

        try {
            //this is the callback to our LoginHandler
            callbackHandler.handle(callbacks);


            //get the username and password provided in the callback
            username = ((NameCallback) callbacks[0]).getName();

            char[] tmpPassword = ((PasswordCallback) callbacks[1]).getPassword();

            if (tmpPassword == null) {
                //treat a NULL password as an empty password
                tmpPassword = new char[0];
            }

            password = new char[tmpPassword.length];
            System.arraycopy(tmpPassword, 0, password, 0, tmpPassword.length);
            ((PasswordCallback) callbacks[1]).clearPassword();
        } catch (java.io.IOException ioe) {
            throw new LoginException(ioe.toString());
        } catch (UnsupportedCallbackException uce) {
            throw new LoginException("Error: " + 
                                     uce.getCallback().toString());
        }


        //verify the username/password
        //insert code to verify user /password here
        //Authentication succeeded
        succeeded = true;

        return true;

        //Authentication failed succeeded =false;
        throw new FailedLoginException("Password Incorrect");
    }

    //the 2 phases of authentication  success commit
    public boolean commit() throws LoginException {
        if (succeeded == false) {
            return false;
        } else {
            //add a Principal (authenticated identity)
            //to the Subject
            //assume the user we authenticated is the WorkPlacePrincipal
            userPrincipal = new WorkPlacePrincipal(username);

            if (!subject.getPrincipals().contains(userPrincipal)) {
                subject.getPrincipals().add(userPrincipal);
            }


            //clean out state
            username = null;

            for (int i = 0; i < password.length; i++)
                password[i] = ' ';

            password = null;
            commitSucceeded = true;

            return true;
        }
    }

    //the 2 phases of authentication  failure abort
    public boolean abort() throws LoginException {
        if (succeeded == false) {
            return false;
        } else if ((succeeded == true) && (commitSucceeded == false)) {
            //login succeeded but overall authentication failed
            succeeded = false;
            username = null;

            if (password != null) {
                for (int i = 0; i < password.length; i++)
                    password[i] = ' ';

                password = null;
            }

            userPrincipal = null;
        } else {
            //overall authentication succeeded and commit succeeded,
            //but someone else s commit failed
            logout();
        }

        return true;
    }

    public boolean logout() throws LoginException {
        subject.getPrincipals().remove(userPrincipal);
        succeeded = false;
        succeeded = commitSucceeded;
        username = null;

        if (password != null) {
            for (int i = 0; i < password.length; i++)
                password[i] = ' ';

            password = null;
        }

        userPrincipal = null;

        return true;
    }
}